from rdflib import Namespace

DCAT = Namespace("http://www.w3.org/ns/dcat#")
OEO = Namespace("http://openenergy-platform.org/ontology/v0.0.1/oeo/")
DCATDE = Namespace("http://dcat-ap.de/def/dcatde/")
SCHEMA = Namespace("http://schema.org/")
SKOS = Namespace("http://www.w3.org/2004/02/skos/core#")
SPDX = Namespace("http://spdx.org/rdf/terms#")
ADMS = Namespace("http://www.w3.org/ns/adms#")
